
import { __decorate, __param } from '@codingame/monaco-vscode-api/external/tslib/tslib.es6';
import { Action } from '@codingame/monaco-vscode-api/vscode/vs/base/common/actions';
import { isCancellationError, getErrorMessage } from '@codingame/monaco-vscode-api/vscode/vs/base/common/errors';
import { Event } from '@codingame/monaco-vscode-api/vscode/vs/base/common/event';
import { Disposable, MutableDisposable, toDisposable, DisposableStore } from '@codingame/monaco-vscode-api/vscode/vs/base/common/lifecycle';
import { isEqual } from '@codingame/monaco-vscode-api/vscode/vs/base/common/resources';
import { URI } from '@codingame/monaco-vscode-api/vscode/vs/base/common/uri';
import { IModelService } from '@codingame/monaco-vscode-api/vscode/vs/editor/common/services/model.service';
import { ILanguageService } from '@codingame/monaco-vscode-api/vscode/vs/editor/common/languages/language.service';
import { ITextModelService } from '@codingame/monaco-vscode-api/vscode/vs/editor/common/services/resolverService.service';
import { localize2, localize } from '@codingame/monaco-vscode-api/vscode/vs/nls';
import { registerAction2, Action2, MenuId, MenuRegistry } from '@codingame/monaco-vscode-api/vscode/vs/platform/actions/common/actions';
import { ICommandService } from '@codingame/monaco-vscode-api/vscode/vs/platform/commands/common/commands.service';
import { RawContextKey, ContextKeyExpr, ContextKeyTrueExpr } from '@codingame/monaco-vscode-api/vscode/vs/platform/contextkey/common/contextkey';
import { IContextKeyService } from '@codingame/monaco-vscode-api/vscode/vs/platform/contextkey/common/contextkey.service';
import { IDialogService } from '@codingame/monaco-vscode-api/vscode/vs/platform/dialogs/common/dialogs.service';
import { IInstantiationService } from '@codingame/monaco-vscode-api/vscode/vs/platform/instantiation/common/instantiation';
import '@codingame/monaco-vscode-api/vscode/vs/platform/notification/common/notification';
import { INotificationService } from '@codingame/monaco-vscode-api/vscode/vs/platform/notification/common/notification.service';
import { IQuickInputService } from '@codingame/monaco-vscode-api/vscode/vs/platform/quickinput/common/quickInput.service';
import { ITelemetryService } from '@codingame/monaco-vscode-api/vscode/vs/platform/telemetry/common/telemetry.service';
import { SyncStatus, registerConfiguration, USER_DATA_SYNC_SCHEME, UserDataSyncErrorCode, SyncResource, UserDataSyncError, USER_DATA_SYNC_LOG_ID } from '@codingame/monaco-vscode-api/vscode/vs/platform/userDataSync/common/userDataSync';
import { IUserDataSyncEnablementService, IUserDataSyncService, IUserDataAutoSyncService, IUserDataSyncStoreManagementService } from '@codingame/monaco-vscode-api/vscode/vs/platform/userDataSync/common/userDataSync.service';
import { EditorResourceAccessor, SideBySideEditor } from '@codingame/monaco-vscode-api/vscode/vs/workbench/common/editor';
import { IOutputService } from '@codingame/monaco-vscode-api/vscode/vs/workbench/services/output/common/output.service';
import { NumberBadge, ProgressBadge } from '@codingame/monaco-vscode-a7c9ae3c-16d2-5d17-86b2-981be7094566-common/vscode/vs/workbench/services/activity/common/activity';
import { IActivityService } from '@codingame/monaco-vscode-api/vscode/vs/workbench/services/activity/common/activity.service';
import { IEditorService } from '@codingame/monaco-vscode-api/vscode/vs/workbench/services/editor/common/editorService.service';
import { IPreferencesService } from '@codingame/monaco-vscode-api/vscode/vs/workbench/services/preferences/common/preferences.service';
import { fromNow } from '@codingame/monaco-vscode-api/vscode/vs/base/common/date';
import { IProductService } from '@codingame/monaco-vscode-api/vscode/vs/platform/product/common/productService.service';
import { IOpenerService } from '@codingame/monaco-vscode-api/vscode/vs/platform/opener/common/opener.service';
import { IAuthenticationService } from '@codingame/monaco-vscode-api/vscode/vs/workbench/services/authentication/common/authentication.service';
import { Registry } from '@codingame/monaco-vscode-api/vscode/vs/platform/registry/common/platform';
import { SyncDescriptor } from '@codingame/monaco-vscode-api/vscode/vs/platform/instantiation/common/descriptors';
import { Extensions, ViewContainerLocation } from '@codingame/monaco-vscode-api/vscode/vs/workbench/common/views';
import { UserDataSyncDataViews } from './userDataSyncViews.js';
import { CONFIGURE_SYNC_COMMAND_ID, getSyncAreaLabel, SHOW_SYNC_LOG_COMMAND_ID, SYNC_TITLE, AccountStatus, CONTEXT_SYNC_STATE, CONTEXT_SYNC_ENABLEMENT, SYNC_VIEW_CONTAINER_ID, CONTEXT_ACCOUNT_STATE, CONTEXT_HAS_CONFLICTS, DOWNLOAD_ACTIVITY_ACTION_DESCRIPTOR, SYNC_VIEW_ICON } from '@codingame/monaco-vscode-api/vscode/vs/workbench/services/userDataSync/common/userDataSync';
import { IUserDataSyncWorkbenchService } from '@codingame/monaco-vscode-api/vscode/vs/workbench/services/userDataSync/common/userDataSync.service';
import { Codicon } from '@codingame/monaco-vscode-api/vscode/vs/base/common/codicons';
import { ViewPaneContainer } from '@codingame/monaco-vscode-12c5f9eb-72d3-57ca-babd-5bef7aa9de3b-common/vscode/vs/workbench/browser/parts/views/viewPaneContainer';
import { Categories } from '@codingame/monaco-vscode-api/vscode/vs/platform/action/common/actionCommonCategories';
import { IHostService } from '@codingame/monaco-vscode-api/vscode/vs/workbench/services/host/browser/host.service';
import { ITextFileService } from '@codingame/monaco-vscode-api/vscode/vs/workbench/services/textfile/common/textfiles.service';
import { ctxIsMergeResultEditor, ctxMergeBaseUri } from '@codingame/monaco-vscode-897bebad-39df-57cb-8a57-36a271d038be-common/vscode/vs/workbench/contrib/mergeEditor/common/mergeEditor';
import { IWorkbenchIssueService } from '@codingame/monaco-vscode-api/vscode/vs/workbench/contrib/issue/common/issue.service';
import { IUserDataProfileService } from '@codingame/monaco-vscode-api/vscode/vs/workbench/services/userDataProfile/common/userDataProfile.service';
import { isWeb } from '@codingame/monaco-vscode-api/vscode/vs/base/common/platform';
import { PromptsConfig } from '@codingame/monaco-vscode-52bb4d5b-ba1a-57fd-9bee-b28824214eac-common/vscode/vs/platform/prompts/common/config';
import { IConfigurationService } from '@codingame/monaco-vscode-api/vscode/vs/platform/configuration/common/configuration.service';
import Severity from '@codingame/monaco-vscode-api/vscode/vs/base/common/severity';

const turnOffSyncCommand = { id: 'workbench.userDataSync.actions.turnOff', title: ( localize2(10937, 'Turn Off')) };
const configureSyncCommand = { id: CONFIGURE_SYNC_COMMAND_ID, title: ( localize2(10938, 'Configure...')) };
const showConflictsCommandId = 'workbench.userDataSync.actions.showConflicts';
const syncNowCommand = {
    id: 'workbench.userDataSync.actions.syncNow',
    title: ( localize2(10939, 'Sync Now')),
    description(userDataSyncService) {
        if (userDataSyncService.status === SyncStatus.Syncing) {
            return localize(10940, "syncing");
        }
        if (userDataSyncService.lastSyncTime) {
            return localize(10941, "synced {0}", fromNow(userDataSyncService.lastSyncTime, true));
        }
        return undefined;
    }
};
const showSyncSettingsCommand = { id: 'workbench.userDataSync.actions.settings', title: ( localize2(10942, 'Show Settings')), };
const showSyncedDataCommand = { id: 'workbench.userDataSync.actions.showSyncedData', title: ( localize2(10943, 'Show Synced Data')), };
const CONTEXT_TURNING_ON_STATE = ( new RawContextKey('userDataSyncTurningOn', false));
let UserDataSyncWorkbenchContribution = class UserDataSyncWorkbenchContribution extends Disposable {
    constructor(userDataSyncEnablementService, userDataSyncService, userDataSyncWorkbenchService, contextKeyService, activityService, notificationService, editorService, userDataProfileService, dialogService, quickInputService, instantiationService, outputService, userDataAutoSyncService, textModelResolverService, preferencesService, telemetryService, productService, openerService, authenticationService, userDataSyncStoreManagementService, hostService, commandService, workbenchIssueService, configService) {
        super();
        this.userDataSyncEnablementService = userDataSyncEnablementService;
        this.userDataSyncService = userDataSyncService;
        this.userDataSyncWorkbenchService = userDataSyncWorkbenchService;
        this.activityService = activityService;
        this.notificationService = notificationService;
        this.editorService = editorService;
        this.userDataProfileService = userDataProfileService;
        this.dialogService = dialogService;
        this.quickInputService = quickInputService;
        this.instantiationService = instantiationService;
        this.outputService = outputService;
        this.preferencesService = preferencesService;
        this.telemetryService = telemetryService;
        this.productService = productService;
        this.openerService = openerService;
        this.authenticationService = authenticationService;
        this.userDataSyncStoreManagementService = userDataSyncStoreManagementService;
        this.hostService = hostService;
        this.commandService = commandService;
        this.workbenchIssueService = workbenchIssueService;
        this.configService = configService;
        this.globalActivityBadgeDisposable = this._register(( new MutableDisposable()));
        this.accountBadgeDisposable = this._register(( new MutableDisposable()));
        this.conflictsDisposables = ( new Map());
        this.invalidContentErrorDisposables = ( new Map());
        this.conflictsActionDisposable = this._register(( new MutableDisposable()));
        this.turningOnSyncContext = CONTEXT_TURNING_ON_STATE.bindTo(contextKeyService);
        if (userDataSyncWorkbenchService.enabled) {
            registerConfiguration();
            this.updateAccountBadge();
            this.updateGlobalActivityBadge();
            this.onDidChangeConflicts(this.userDataSyncService.conflicts);
            this._register(Event.any(Event.debounce(userDataSyncService.onDidChangeStatus, () => undefined, 500), this.userDataSyncEnablementService.onDidChangeEnablement, this.userDataSyncWorkbenchService.onDidChangeAccountStatus)(() => {
                this.updateAccountBadge();
                this.updateGlobalActivityBadge();
            }));
            this._register(userDataSyncService.onDidChangeConflicts(() => this.onDidChangeConflicts(this.userDataSyncService.conflicts)));
            this._register(userDataSyncEnablementService.onDidChangeEnablement(() => this.onDidChangeConflicts(this.userDataSyncService.conflicts)));
            this._register(userDataSyncService.onSyncErrors(errors => this.onSynchronizerErrors(errors)));
            this._register(userDataAutoSyncService.onError(error => this.onAutoSyncError(error)));
            this.registerActions();
            this.registerViews();
            textModelResolverService.registerTextModelContentProvider(USER_DATA_SYNC_SCHEME, instantiationService.createInstance(UserDataRemoteContentProvider));
            this._register(Event.any(userDataSyncService.onDidChangeStatus, userDataSyncEnablementService.onDidChangeEnablement)(() => this.turningOnSync = !userDataSyncEnablementService.isEnabled() && userDataSyncService.status !== SyncStatus.Idle));
        }
    }
    get turningOnSync() {
        return !!this.turningOnSyncContext.get();
    }
    set turningOnSync(turningOn) {
        this.turningOnSyncContext.set(turningOn);
        this.updateGlobalActivityBadge();
    }
    toKey({ syncResource: resource, profile }) {
        return `${profile.id}:${resource}`;
    }
    onDidChangeConflicts(conflicts) {
        this.updateGlobalActivityBadge();
        this.registerShowConflictsAction();
        if (!this.userDataSyncEnablementService.isEnabled()) {
            return;
        }
        if (conflicts.length) {
            for (const [key, disposable] of this.conflictsDisposables.entries()) {
                if (!( conflicts.some(conflict => this.toKey(conflict) === key))) {
                    disposable.dispose();
                    this.conflictsDisposables.delete(key);
                }
            }
            for (const conflict of this.userDataSyncService.conflicts) {
                const key = this.toKey(conflict);
                if (!( this.conflictsDisposables.has(key))) {
                    const conflictsArea = getSyncAreaLabel(conflict.syncResource);
                    const handle = this.notificationService.prompt(Severity.Warning, ( localize(
                        10944,
                        "Unable to sync due to conflicts in {0}. Please resolve them to continue.",
                        conflictsArea.toLowerCase()
                    )), [
                        {
                            label: ( localize(10945, "Replace Remote")),
                            run: () => {
                                this.acceptLocal(conflict, conflict.conflicts[0]);
                            }
                        },
                        {
                            label: ( localize(10946, "Replace Local")),
                            run: () => {
                                this.acceptRemote(conflict, conflict.conflicts[0]);
                            }
                        },
                        {
                            label: ( localize(10947, "Show Conflicts")),
                            run: () => {
                                this.telemetryService.publicLog2('sync/showConflicts', { source: conflict.syncResource });
                                this.userDataSyncWorkbenchService.showConflicts(conflict.conflicts[0]);
                            }
                        }
                    ], {
                        sticky: true
                    });
                    this.conflictsDisposables.set(key, toDisposable(() => {
                        handle.close();
                        this.conflictsDisposables.delete(key);
                    }));
                }
            }
        }
        else {
            this.conflictsDisposables.forEach(disposable => disposable.dispose());
            this.conflictsDisposables.clear();
        }
    }
    async acceptRemote(syncResource, conflict) {
        try {
            await this.userDataSyncService.accept(syncResource, conflict.remoteResource, undefined, this.userDataSyncEnablementService.isEnabled());
        }
        catch (e) {
            this.notificationService.error(( localize(
                10948,
                "Error while accepting changes. Please check [logs]({0}) for more details.",
                `command:${SHOW_SYNC_LOG_COMMAND_ID}`
            )));
        }
    }
    async acceptLocal(syncResource, conflict) {
        try {
            await this.userDataSyncService.accept(syncResource, conflict.localResource, undefined, this.userDataSyncEnablementService.isEnabled());
        }
        catch (e) {
            this.notificationService.error(( localize(
                10948,
                "Error while accepting changes. Please check [logs]({0}) for more details.",
                `command:${SHOW_SYNC_LOG_COMMAND_ID}`
            )));
        }
    }
    onAutoSyncError(error) {
        switch (error.code) {
            case UserDataSyncErrorCode.SessionExpired:
                this.notificationService.notify({
                    severity: Severity.Info,
                    message: ( localize(
                        10949,
                        "Settings sync was turned off because current session is expired, please sign in again to turn on sync."
                    )),
                    actions: {
                        primary: [( new Action('turn on sync', ( localize(10950, "Turn on Settings Sync...")), undefined, true, () => this.turnOn()))]
                    }
                });
                break;
            case UserDataSyncErrorCode.TurnedOff:
                this.notificationService.notify({
                    severity: Severity.Info,
                    message: ( localize(
                        10951,
                        "Settings sync was turned off from another device, please turn on sync again."
                    )),
                    actions: {
                        primary: [( new Action('turn on sync', ( localize(10950, "Turn on Settings Sync...")), undefined, true, () => this.turnOn()))]
                    }
                });
                break;
            case UserDataSyncErrorCode.TooLarge:
                if (error.resource === SyncResource.Keybindings || error.resource === SyncResource.Settings || error.resource === SyncResource.Tasks) {
                    this.disableSync(error.resource);
                    const sourceArea = getSyncAreaLabel(error.resource);
                    this.handleTooLargeError(error.resource, ( localize(
                        10952,
                        "Disabled syncing {0} because size of the {1} file to sync is larger than {2}. Please open the file and reduce the size and enable sync",
                        sourceArea.toLowerCase(),
                        sourceArea.toLowerCase(),
                        '100kb'
                    )), error);
                }
                break;
            case UserDataSyncErrorCode.LocalTooManyProfiles:
                this.disableSync(SyncResource.Profiles);
                this.notificationService.error(( localize(
                    10953,
                    "Disabled syncing profiles because there are too many profiles to sync. Settings Sync supports syncing maximum 20 profiles. Please reduce the number of profiles and enable sync"
                )));
                break;
            case UserDataSyncErrorCode.IncompatibleLocalContent:
            case UserDataSyncErrorCode.Gone:
            case UserDataSyncErrorCode.UpgradeRequired: {
                const message = ( localize(
                    10954,
                    "Settings sync is disabled because the current version ({0}, {1}) is not compatible with the sync service. Please update before turning on sync.",
                    this.productService.version,
                    this.productService.commit
                ));
                const operationId = error.operationId ? ( localize(10955, "Operation Id: {0}", error.operationId)) : undefined;
                this.notificationService.notify({
                    severity: Severity.Error,
                    message: operationId ? `${message} ${operationId}` : message,
                });
                break;
            }
            case UserDataSyncErrorCode.MethodNotFound: {
                const message = ( localize(
                    10956,
                    "Settings sync is disabled because the client is making invalid requests. Please report an issue with the logs."
                ));
                const operationId = error.operationId ? ( localize(10955, "Operation Id: {0}", error.operationId)) : undefined;
                this.notificationService.notify({
                    severity: Severity.Error,
                    message: operationId ? `${message} ${operationId}` : message,
                    actions: {
                        primary: [
                            ( new Action('Show Sync Logs', ( localize(10957, "Show Log")), undefined, true, () => this.commandService.executeCommand(SHOW_SYNC_LOG_COMMAND_ID))),
                            ( new Action('Report Issue', ( localize(10958, "Report Issue")), undefined, true, () => this.workbenchIssueService.openReporter()))
                        ]
                    }
                });
                break;
            }
            case UserDataSyncErrorCode.IncompatibleRemoteContent:
                this.notificationService.notify({
                    severity: Severity.Error,
                    message: ( localize(
                        10959,
                        "Settings sync is disabled because your data in the cloud is older than that of the client. Please clear your data in the cloud before turning on sync."
                    )),
                    actions: {
                        primary: [
                            ( new Action('reset', ( localize(10960, "Clear Data in Cloud...")), undefined, true, () => this.userDataSyncWorkbenchService.resetSyncedData())),
                            ( new Action('show synced data', ( localize(10961, "Show Synced Data")), undefined, true, () => this.userDataSyncWorkbenchService.showSyncActivity()))
                        ]
                    }
                });
                return;
            case UserDataSyncErrorCode.ServiceChanged:
                this.notificationService.notify({
                    severity: Severity.Info,
                    message: this.userDataSyncStoreManagementService.userDataSyncStore?.type === 'insiders' ?
                        ( localize(10962, "Settings Sync has been switched to insiders service")) :
                        ( localize(10963, "Settings Sync has been switched to stable service")),
                });
                return;
            case UserDataSyncErrorCode.DefaultServiceChanged:
                if (this.userDataSyncEnablementService.isEnabled()) {
                    this.notificationService.notify({
                        severity: Severity.Info,
                        message: ( localize(
                            10964,
                            "Settings sync now uses a separate service, more information is available in the [Settings Sync Documentation](https://aka.ms/vscode-settings-sync-help#_syncing-stable-versus-insiders)."
                        )),
                    });
                }
                else {
                    this.notificationService.notify({
                        severity: Severity.Info,
                        message: ( localize(
                            10965,
                            "Settings sync was turned off because {0} now uses a separate service. Please turn on sync again.",
                            this.productService.nameLong
                        )),
                        actions: {
                            primary: [( new Action('turn on sync', ( localize(10950, "Turn on Settings Sync...")), undefined, true, () => this.turnOn()))]
                        }
                    });
                }
                return;
        }
    }
    handleTooLargeError(resource, message, error) {
        const operationId = error.operationId ? ( localize(10955, "Operation Id: {0}", error.operationId)) : undefined;
        this.notificationService.notify({
            severity: Severity.Error,
            message: operationId ? `${message} ${operationId}` : message,
            actions: {
                primary: [( new Action('open sync file', ( localize(10966, "Open {0} File", getSyncAreaLabel(resource))), undefined, true, () => resource === SyncResource.Settings ? this.preferencesService.openUserSettings({ jsonEditor: true }) : this.preferencesService.openGlobalKeybindingSettings(true)))]
            }
        });
    }
    onSynchronizerErrors(errors) {
        if (errors.length) {
            for (const { profile, syncResource: resource, error } of errors) {
                switch (error.code) {
                    case UserDataSyncErrorCode.LocalInvalidContent:
                        this.handleInvalidContentError({ profile, syncResource: resource });
                        break;
                    default: {
                        const key = `${profile.id}:${resource}`;
                        const disposable = this.invalidContentErrorDisposables.get(key);
                        if (disposable) {
                            disposable.dispose();
                            this.invalidContentErrorDisposables.delete(key);
                        }
                    }
                }
            }
        }
        else {
            this.invalidContentErrorDisposables.forEach(disposable => disposable.dispose());
            this.invalidContentErrorDisposables.clear();
        }
    }
    handleInvalidContentError({ profile, syncResource: source }) {
        if (this.userDataProfileService.currentProfile.id !== profile.id) {
            return;
        }
        const key = `${profile.id}:${source}`;
        if (( this.invalidContentErrorDisposables.has(key))) {
            return;
        }
        if (source !== SyncResource.Settings && source !== SyncResource.Keybindings && source !== SyncResource.Tasks) {
            return;
        }
        if (!this.hostService.hasFocus) {
            return;
        }
        const resource = source === SyncResource.Settings ? this.userDataProfileService.currentProfile.settingsResource
            : source === SyncResource.Keybindings ? this.userDataProfileService.currentProfile.keybindingsResource
                : this.userDataProfileService.currentProfile.tasksResource;
        const editorUri = EditorResourceAccessor.getCanonicalUri(this.editorService.activeEditor, { supportSideBySide: SideBySideEditor.PRIMARY });
        if (isEqual(resource, editorUri)) {
            return;
        }
        const errorArea = getSyncAreaLabel(source);
        const handle = this.notificationService.notify({
            severity: Severity.Error,
            message: ( localize(
                10967,
                "Unable to sync {0} because the content in the file is not valid. Please open the file and correct it.",
                errorArea.toLowerCase()
            )),
            actions: {
                primary: [( new Action('open sync file', ( localize(10966, "Open {0} File", errorArea)), undefined, true, () => source === SyncResource.Settings ? this.preferencesService.openUserSettings({ jsonEditor: true }) : this.preferencesService.openGlobalKeybindingSettings(true)))]
            }
        });
        this.invalidContentErrorDisposables.set(key, toDisposable(() => {
            handle.close();
            this.invalidContentErrorDisposables.delete(key);
        }));
    }
    getConflictsCount() {
        return this.userDataSyncService.conflicts.reduce((result, { conflicts }) => { return result + conflicts.length; }, 0);
    }
    async updateGlobalActivityBadge() {
        this.globalActivityBadgeDisposable.clear();
        let badge = undefined;
        if (this.userDataSyncService.conflicts.length && this.userDataSyncEnablementService.isEnabled()) {
            badge = ( new NumberBadge(this.getConflictsCount(), () => ( localize(10968, "{0}: Conflicts Detected", SYNC_TITLE.value))));
        }
        else if (this.turningOnSync) {
            badge = ( new ProgressBadge(() => ( localize(10969, "Turning on Settings Sync..."))));
        }
        if (badge) {
            this.globalActivityBadgeDisposable.value = this.activityService.showGlobalActivity({ badge });
        }
    }
    async updateAccountBadge() {
        this.accountBadgeDisposable.clear();
        let badge = undefined;
        if (this.userDataSyncService.status !== SyncStatus.Uninitialized && this.userDataSyncEnablementService.isEnabled() && this.userDataSyncWorkbenchService.accountStatus === AccountStatus.Unavailable) {
            badge = ( new NumberBadge(1, () => ( localize(10970, "Sign in to Sync Settings"))));
        }
        if (badge) {
            this.accountBadgeDisposable.value = this.activityService.showAccountsActivity({ badge });
        }
    }
    async turnOn() {
        try {
            if (!this.userDataSyncWorkbenchService.authenticationProviders.length) {
                throw ( new Error(( localize(10971, "No authentication providers are available."))));
            }
            const turnOn = await this.askToConfigure();
            if (!turnOn) {
                return;
            }
            if (this.userDataSyncStoreManagementService.userDataSyncStore?.canSwitch) {
                await this.selectSettingsSyncService(this.userDataSyncStoreManagementService.userDataSyncStore);
            }
            await this.userDataSyncWorkbenchService.turnOn();
        }
        catch (e) {
            if (isCancellationError(e)) {
                return;
            }
            if (e instanceof UserDataSyncError) {
                switch (e.code) {
                    case UserDataSyncErrorCode.TooLarge:
                        if (e.resource === SyncResource.Keybindings || e.resource === SyncResource.Settings || e.resource === SyncResource.Tasks) {
                            this.handleTooLargeError(e.resource, ( localize(
                                10972,
                                "Settings sync cannot be turned on because size of the {0} file to sync is larger than {1}. Please open the file and reduce the size and turn on sync",
                                getSyncAreaLabel(e.resource).toLowerCase(),
                                '100kb'
                            )), e);
                            return;
                        }
                        break;
                    case UserDataSyncErrorCode.IncompatibleLocalContent:
                    case UserDataSyncErrorCode.Gone:
                    case UserDataSyncErrorCode.UpgradeRequired: {
                        const message = ( localize(
                            10973,
                            "Settings sync cannot be turned on because the current version ({0}, {1}) is not compatible with the sync service. Please update before turning on sync.",
                            this.productService.version,
                            this.productService.commit
                        ));
                        const operationId = e.operationId ? ( localize(10955, "Operation Id: {0}", e.operationId)) : undefined;
                        this.notificationService.notify({
                            severity: Severity.Error,
                            message: operationId ? `${message} ${operationId}` : message,
                        });
                        return;
                    }
                    case UserDataSyncErrorCode.IncompatibleRemoteContent:
                        this.notificationService.notify({
                            severity: Severity.Error,
                            message: ( localize(
                                10974,
                                "Settings sync cannot be turned on because your data in the cloud is older than that of the client. Please clear your data in the cloud before turning on sync."
                            )),
                            actions: {
                                primary: [
                                    ( new Action('reset', ( localize(10960, "Clear Data in Cloud...")), undefined, true, () => this.userDataSyncWorkbenchService.resetSyncedData())),
                                    ( new Action('show synced data', ( localize(10961, "Show Synced Data")), undefined, true, () => this.userDataSyncWorkbenchService.showSyncActivity()))
                                ]
                            }
                        });
                        return;
                    case UserDataSyncErrorCode.Unauthorized:
                    case UserDataSyncErrorCode.Forbidden:
                        this.notificationService.error(( localize(10975, "Error while turning on Settings Sync: Authentication failed.")));
                        return;
                }
                this.notificationService.error(( localize(
                    10976,
                    "Error while turning on Settings Sync. Please check [logs]({0}) for more details.",
                    `command:${SHOW_SYNC_LOG_COMMAND_ID}`
                )));
            }
            else {
                this.notificationService.error(( localize(10977, "Error while turning on Settings Sync. {0}", getErrorMessage(e))));
            }
        }
    }
    async askToConfigure() {
        return (new Promise((c, e) => {
            const disposables = ( new DisposableStore());
            const quickPick = this.quickInputService.createQuickPick();
            disposables.add(quickPick);
            quickPick.title = SYNC_TITLE.value;
            quickPick.ok = false;
            quickPick.customButton = true;
            quickPick.customLabel = ( localize(10978, "Sign in"));
            quickPick.description = ( localize(10979, "Please sign in to backup and sync your data across devices."));
            quickPick.canSelectMany = true;
            quickPick.ignoreFocusOut = true;
            quickPick.hideInput = true;
            quickPick.hideCheckAll = true;
            const items = this.getConfigureSyncQuickPickItems();
            quickPick.items = items;
            quickPick.selectedItems = items.filter(item => this.userDataSyncEnablementService.isResourceEnabled(item.id, true));
            let accepted = false;
            disposables.add(Event.any(quickPick.onDidAccept, quickPick.onDidCustom)(() => {
                accepted = true;
                quickPick.hide();
            }));
            disposables.add(quickPick.onDidHide(() => {
                try {
                    if (accepted) {
                        this.updateConfiguration(items, quickPick.selectedItems);
                    }
                    c(accepted);
                }
                catch (error) {
                    e(error);
                }
                finally {
                    disposables.dispose();
                }
            }));
            quickPick.show();
        }));
    }
    getConfigureSyncQuickPickItems() {
        const result = [{
                id: SyncResource.Settings,
                label: getSyncAreaLabel(SyncResource.Settings)
            }, {
                id: SyncResource.Keybindings,
                label: getSyncAreaLabel(SyncResource.Keybindings),
            }, {
                id: SyncResource.Snippets,
                label: getSyncAreaLabel(SyncResource.Snippets)
            }, {
                id: SyncResource.Tasks,
                label: getSyncAreaLabel(SyncResource.Tasks)
            }, {
                id: SyncResource.GlobalState,
                label: getSyncAreaLabel(SyncResource.GlobalState),
            }, {
                id: SyncResource.Extensions,
                label: getSyncAreaLabel(SyncResource.Extensions)
            }, {
                id: SyncResource.Profiles,
                label: getSyncAreaLabel(SyncResource.Profiles),
            }];
        const isInsiders = (this.productService.quality !== 'stable');
        if (PromptsConfig.enabled(this.configService) && isInsiders) {
            result.push({
                id: SyncResource.Prompts,
                label: getSyncAreaLabel(SyncResource.Prompts)
            });
        }
        return result;
    }
    updateConfiguration(items, selectedItems) {
        for (const item of items) {
            const wasEnabled = this.userDataSyncEnablementService.isResourceEnabled(item.id);
            const isEnabled = !!selectedItems.filter(selected => selected.id === item.id)[0];
            if (wasEnabled !== isEnabled) {
                this.userDataSyncEnablementService.setResourceEnablement(item.id, isEnabled);
            }
        }
    }
    async configureSyncOptions() {
        return (new Promise((c, e) => {
            const disposables = ( new DisposableStore());
            const quickPick = this.quickInputService.createQuickPick();
            disposables.add(quickPick);
            quickPick.title = ( localize(10980, "{0}: Configure...", SYNC_TITLE.value));
            quickPick.placeholder = ( localize(10981, "Choose what to sync"));
            quickPick.canSelectMany = true;
            quickPick.ignoreFocusOut = true;
            quickPick.ok = true;
            const items = this.getConfigureSyncQuickPickItems();
            quickPick.items = items;
            quickPick.selectedItems = items.filter(item => this.userDataSyncEnablementService.isResourceEnabled(item.id));
            disposables.add(quickPick.onDidAccept(async () => {
                if (quickPick.selectedItems.length) {
                    this.updateConfiguration(items, quickPick.selectedItems);
                    quickPick.hide();
                }
            }));
            disposables.add(quickPick.onDidHide(() => {
                disposables.dispose();
                c();
            }));
            quickPick.show();
        }));
    }
    async turnOff() {
        const result = await this.dialogService.confirm({
            message: ( localize(10982, "Do you want to turn off sync?")),
            detail: ( localize(
                10983,
                "Your settings, keybindings, extensions, snippets and UI State will no longer be synced."
            )),
            primaryButton: ( localize(10984, "&&Turn off")),
            checkbox: this.userDataSyncWorkbenchService.accountStatus === AccountStatus.Available ? {
                label: ( localize(
                    10985,
                    "Turn off sync on all your devices and clear the data from the cloud."
                ))
            } : undefined
        });
        if (result.confirmed) {
            return this.userDataSyncWorkbenchService.turnoff(!!result.checkboxChecked);
        }
    }
    disableSync(source) {
        switch (source) {
            case SyncResource.Settings: return this.userDataSyncEnablementService.setResourceEnablement(SyncResource.Settings, false);
            case SyncResource.Keybindings: return this.userDataSyncEnablementService.setResourceEnablement(SyncResource.Keybindings, false);
            case SyncResource.Snippets: return this.userDataSyncEnablementService.setResourceEnablement(SyncResource.Snippets, false);
            case SyncResource.Tasks: return this.userDataSyncEnablementService.setResourceEnablement(SyncResource.Tasks, false);
            case SyncResource.Extensions: return this.userDataSyncEnablementService.setResourceEnablement(SyncResource.Extensions, false);
            case SyncResource.GlobalState: return this.userDataSyncEnablementService.setResourceEnablement(SyncResource.GlobalState, false);
            case SyncResource.Profiles: return this.userDataSyncEnablementService.setResourceEnablement(SyncResource.Profiles, false);
        }
    }
    showSyncActivity() {
        return this.outputService.showChannel(USER_DATA_SYNC_LOG_ID);
    }
    async selectSettingsSyncService(userDataSyncStore) {
        return (new Promise((c, e) => {
            const disposables = ( new DisposableStore());
            const quickPick = disposables.add(this.quickInputService.createQuickPick());
            quickPick.title = ( localize(10986, "{0}: Select Service", SYNC_TITLE.value));
            quickPick.description = ( localize(
                10987,
                "Ensure you are using the same settings sync service when syncing with multiple environments"
            ));
            quickPick.hideInput = true;
            quickPick.ignoreFocusOut = true;
            const getDescription = (url) => {
                const isDefault = isEqual(url, userDataSyncStore.defaultUrl);
                if (isDefault) {
                    return localize(10988, "Default");
                }
                return undefined;
            };
            quickPick.items = [
                {
                    id: 'insiders',
                    label: ( localize(10989, "Insiders")),
                    description: getDescription(userDataSyncStore.insidersUrl)
                },
                {
                    id: 'stable',
                    label: ( localize(10990, "Stable")),
                    description: getDescription(userDataSyncStore.stableUrl)
                }
            ];
            disposables.add(quickPick.onDidAccept(async () => {
                try {
                    await this.userDataSyncStoreManagementService.switch(quickPick.selectedItems[0].id);
                    c();
                }
                catch (error) {
                    e(error);
                }
                finally {
                    quickPick.hide();
                }
            }));
            disposables.add(quickPick.onDidHide(() => disposables.dispose()));
            quickPick.show();
        }));
    }
    registerActions() {
        if (this.userDataSyncEnablementService.canToggleEnablement()) {
            this.registerTurnOnSyncAction();
            this.registerTurnOffSyncAction();
        }
        this.registerTurningOnSyncAction();
        this.registerCancelTurnOnSyncAction();
        this.registerSignInAction();
        this.registerShowConflictsAction();
        this.registerEnableSyncViewsAction();
        this.registerManageSyncAction();
        this.registerSyncNowAction();
        this.registerConfigureSyncAction();
        this.registerShowSettingsAction();
        this.registerHelpAction();
        this.registerShowLogAction();
        this.registerResetSyncDataAction();
        this.registerAcceptMergesAction();
        if (isWeb) {
            this.registerDownloadSyncActivityAction();
        }
    }
    registerTurnOnSyncAction() {
        const that = this;
        const when = ( ContextKeyExpr.and(( CONTEXT_SYNC_STATE.notEqualsTo(SyncStatus.Uninitialized)), ( CONTEXT_SYNC_ENABLEMENT.toNegated()), ( CONTEXT_TURNING_ON_STATE.negate())));
        this._register(registerAction2(class TurningOnSyncAction extends Action2 {
            constructor() {
                super({
                    id: 'workbench.userDataSync.actions.turnOn',
                    title: ( localize2(10991, 'Backup and Sync Settings...')),
                    category: SYNC_TITLE,
                    f1: true,
                    precondition: when,
                    menu: [{
                            group: '3_configuration',
                            id: MenuId.GlobalActivity,
                            when,
                            order: 2
                        }, {
                            group: '3_configuration',
                            id: MenuId.MenubarPreferencesMenu,
                            when,
                            order: 2
                        }, {
                            group: '1_settings',
                            id: MenuId.AccountsContext,
                            when,
                            order: 2
                        }]
                });
            }
            async run() {
                return that.turnOn();
            }
        }));
    }
    registerTurningOnSyncAction() {
        const when = ( ContextKeyExpr.and(( CONTEXT_SYNC_STATE.notEqualsTo(SyncStatus.Uninitialized)), ( CONTEXT_SYNC_ENABLEMENT.toNegated()), CONTEXT_TURNING_ON_STATE));
        this._register(registerAction2(class TurningOnSyncAction extends Action2 {
            constructor() {
                super({
                    id: 'workbench.userData.actions.turningOn',
                    title: ( localize(10992, "Turning on Settings Sync...")),
                    precondition: ( ContextKeyExpr.false()),
                    menu: [{
                            group: '3_configuration',
                            id: MenuId.GlobalActivity,
                            when,
                            order: 2
                        }, {
                            group: '1_settings',
                            id: MenuId.AccountsContext,
                            when,
                        }]
                });
            }
            async run() { }
        }));
    }
    registerCancelTurnOnSyncAction() {
        const that = this;
        this._register(registerAction2(class TurningOnSyncAction extends Action2 {
            constructor() {
                super({
                    id: 'workbench.userData.actions.cancelTurnOn',
                    title: ( localize(10993, "Cancel")),
                    icon: Codicon.stopCircle,
                    menu: {
                        id: MenuId.ViewContainerTitle,
                        when: ( ContextKeyExpr.and(CONTEXT_TURNING_ON_STATE, ( ContextKeyExpr.equals('viewContainer', SYNC_VIEW_CONTAINER_ID)))),
                        group: 'navigation',
                        order: 1
                    }
                });
            }
            async run() {
                return that.userDataSyncWorkbenchService.turnoff(false);
            }
        }));
    }
    registerSignInAction() {
        const that = this;
        const id = 'workbench.userData.actions.signin';
        const when = ( ContextKeyExpr.and(( CONTEXT_SYNC_STATE.notEqualsTo(SyncStatus.Uninitialized)), CONTEXT_SYNC_ENABLEMENT, ( CONTEXT_ACCOUNT_STATE.isEqualTo(AccountStatus.Unavailable))));
        this._register(registerAction2(class StopSyncAction extends Action2 {
            constructor() {
                super({
                    id: 'workbench.userData.actions.signin',
                    title: ( localize(10994, "Sign in to Sync Settings")),
                    menu: {
                        group: '3_configuration',
                        id: MenuId.GlobalActivity,
                        when,
                        order: 2
                    }
                });
            }
            async run() {
                try {
                    await that.userDataSyncWorkbenchService.signIn();
                }
                catch (e) {
                    that.notificationService.error(e);
                }
            }
        }));
        this._register(( MenuRegistry.appendMenuItem(MenuId.AccountsContext, {
            group: '1_settings',
            command: {
                id,
                title: ( localize(10995, "Sign in to Sync Settings (1)")),
            },
            when
        })));
    }
    getShowConflictsTitle() {
        return localize2(10996, "Show Conflicts ({0})", this.getConflictsCount());
    }
    registerShowConflictsAction() {
        this.conflictsActionDisposable.value = undefined;
        const that = this;
        this.conflictsActionDisposable.value = registerAction2(class TurningOnSyncAction extends Action2 {
            constructor() {
                super({
                    id: showConflictsCommandId,
                    get title() { return that.getShowConflictsTitle(); },
                    category: SYNC_TITLE,
                    f1: true,
                    precondition: CONTEXT_HAS_CONFLICTS,
                    menu: [{
                            group: '3_configuration',
                            id: MenuId.GlobalActivity,
                            when: CONTEXT_HAS_CONFLICTS,
                            order: 2
                        }, {
                            group: '3_configuration',
                            id: MenuId.MenubarPreferencesMenu,
                            when: CONTEXT_HAS_CONFLICTS,
                            order: 2
                        }]
                });
            }
            async run() {
                return that.userDataSyncWorkbenchService.showConflicts();
            }
        });
    }
    registerManageSyncAction() {
        const that = this;
        const when = ( ContextKeyExpr.and(CONTEXT_SYNC_ENABLEMENT, ( CONTEXT_ACCOUNT_STATE.notEqualsTo(AccountStatus.Unavailable)), ( CONTEXT_SYNC_STATE.notEqualsTo(SyncStatus.Uninitialized))));
        this._register(registerAction2(class SyncStatusAction extends Action2 {
            constructor() {
                super({
                    id: 'workbench.userDataSync.actions.manage',
                    title: ( localize(10997, "Settings Sync is On")),
                    toggled: ContextKeyTrueExpr.INSTANCE,
                    menu: [
                        {
                            id: MenuId.GlobalActivity,
                            group: '3_configuration',
                            when,
                            order: 2
                        },
                        {
                            id: MenuId.MenubarPreferencesMenu,
                            group: '3_configuration',
                            when,
                            order: 2,
                        },
                        {
                            id: MenuId.AccountsContext,
                            group: '1_settings',
                            when,
                        }
                    ],
                });
            }
            run(accessor) {
                return ( new Promise((c, e) => {
                    const quickInputService = accessor.get(IQuickInputService);
                    const commandService = accessor.get(ICommandService);
                    const disposables = ( new DisposableStore());
                    const quickPick = quickInputService.createQuickPick({ useSeparators: true });
                    disposables.add(quickPick);
                    const items = [];
                    if (that.userDataSyncService.conflicts.length) {
                        items.push({ id: showConflictsCommandId, label: `${SYNC_TITLE.value}: ${that.getShowConflictsTitle().original}` });
                        items.push({ type: 'separator' });
                    }
                    items.push({ id: configureSyncCommand.id, label: `${SYNC_TITLE.value}: ${configureSyncCommand.title.original}` });
                    items.push({ id: showSyncSettingsCommand.id, label: `${SYNC_TITLE.value}: ${showSyncSettingsCommand.title.original}` });
                    items.push({ id: showSyncedDataCommand.id, label: `${SYNC_TITLE.value}: ${showSyncedDataCommand.title.original}` });
                    items.push({ type: 'separator' });
                    items.push({ id: syncNowCommand.id, label: `${SYNC_TITLE.value}: ${syncNowCommand.title.original}`, description: syncNowCommand.description(that.userDataSyncService) });
                    if (that.userDataSyncEnablementService.canToggleEnablement()) {
                        const account = that.userDataSyncWorkbenchService.current;
                        items.push({ id: turnOffSyncCommand.id, label: `${SYNC_TITLE.value}: ${turnOffSyncCommand.title.original}`, description: account ? `${account.accountName} (${that.authenticationService.getProvider(account.authenticationProviderId).label})` : undefined });
                    }
                    quickPick.items = items;
                    disposables.add(quickPick.onDidAccept(() => {
                        if (quickPick.selectedItems[0] && quickPick.selectedItems[0].id) {
                            commandService.executeCommand(quickPick.selectedItems[0].id);
                        }
                        quickPick.hide();
                    }));
                    disposables.add(quickPick.onDidHide(() => {
                        disposables.dispose();
                        c();
                    }));
                    quickPick.show();
                }));
            }
        }));
    }
    registerEnableSyncViewsAction() {
        const that = this;
        const when = ( ContextKeyExpr.and(( CONTEXT_ACCOUNT_STATE.isEqualTo(AccountStatus.Available)), ( CONTEXT_SYNC_STATE.notEqualsTo(SyncStatus.Uninitialized))));
        this._register(registerAction2(class SyncStatusAction extends Action2 {
            constructor() {
                super({
                    id: showSyncedDataCommand.id,
                    title: showSyncedDataCommand.title,
                    category: SYNC_TITLE,
                    precondition: when,
                    menu: {
                        id: MenuId.CommandPalette,
                        when
                    }
                });
            }
            run(accessor) {
                return that.userDataSyncWorkbenchService.showSyncActivity();
            }
        }));
    }
    registerSyncNowAction() {
        const that = this;
        this._register(registerAction2(class SyncNowAction extends Action2 {
            constructor() {
                super({
                    id: syncNowCommand.id,
                    title: syncNowCommand.title,
                    category: SYNC_TITLE,
                    menu: {
                        id: MenuId.CommandPalette,
                        when: ( ContextKeyExpr.and(CONTEXT_SYNC_ENABLEMENT, ( CONTEXT_ACCOUNT_STATE.isEqualTo(AccountStatus.Available)), ( CONTEXT_SYNC_STATE.notEqualsTo(SyncStatus.Uninitialized))))
                    }
                });
            }
            run(accessor) {
                return that.userDataSyncWorkbenchService.syncNow();
            }
        }));
    }
    registerTurnOffSyncAction() {
        const that = this;
        this._register(registerAction2(class StopSyncAction extends Action2 {
            constructor() {
                super({
                    id: turnOffSyncCommand.id,
                    title: turnOffSyncCommand.title,
                    category: SYNC_TITLE,
                    menu: {
                        id: MenuId.CommandPalette,
                        when: ( ContextKeyExpr.and(( CONTEXT_SYNC_STATE.notEqualsTo(SyncStatus.Uninitialized)), CONTEXT_SYNC_ENABLEMENT)),
                    },
                });
            }
            async run() {
                try {
                    await that.turnOff();
                }
                catch (e) {
                    if (!isCancellationError(e)) {
                        that.notificationService.error(( localize(
                            10998,
                            "Error while turning off Settings Sync. Please check [logs]({0}) for more details.",
                            `command:${SHOW_SYNC_LOG_COMMAND_ID}`
                        )));
                    }
                }
            }
        }));
    }
    registerConfigureSyncAction() {
        const that = this;
        const when = ( ContextKeyExpr.and(( CONTEXT_SYNC_STATE.notEqualsTo(SyncStatus.Uninitialized)), CONTEXT_SYNC_ENABLEMENT));
        this._register(registerAction2(class ConfigureSyncAction extends Action2 {
            constructor() {
                super({
                    id: configureSyncCommand.id,
                    title: configureSyncCommand.title,
                    category: SYNC_TITLE,
                    icon: Codicon.settingsGear,
                    tooltip: ( localize(10999, "Configure...")),
                    menu: [{
                            id: MenuId.CommandPalette,
                            when
                        }, {
                            id: MenuId.ViewContainerTitle,
                            when: ( ContextKeyExpr.and(CONTEXT_SYNC_ENABLEMENT, ( ContextKeyExpr.equals('viewContainer', SYNC_VIEW_CONTAINER_ID)))),
                            group: 'navigation',
                            order: 2
                        }]
                });
            }
            run() { return that.configureSyncOptions(); }
        }));
    }
    registerShowLogAction() {
        const that = this;
        this._register(registerAction2(class ShowSyncActivityAction extends Action2 {
            constructor() {
                super({
                    id: SHOW_SYNC_LOG_COMMAND_ID,
                    title: ( localize(11000, "{0}: Show Log", SYNC_TITLE.value)),
                    tooltip: ( localize(11001, "Show Log")),
                    icon: Codicon.output,
                    menu: [{
                            id: MenuId.CommandPalette,
                            when: ( ContextKeyExpr.and(( CONTEXT_SYNC_STATE.notEqualsTo(SyncStatus.Uninitialized)))),
                        }, {
                            id: MenuId.ViewContainerTitle,
                            when: ( ContextKeyExpr.equals('viewContainer', SYNC_VIEW_CONTAINER_ID)),
                            group: 'navigation',
                            order: 1
                        }],
                });
            }
            run() { return that.showSyncActivity(); }
        }));
    }
    registerShowSettingsAction() {
        this._register(registerAction2(class ShowSyncSettingsAction extends Action2 {
            constructor() {
                super({
                    id: showSyncSettingsCommand.id,
                    title: showSyncSettingsCommand.title,
                    category: SYNC_TITLE,
                    menu: {
                        id: MenuId.CommandPalette,
                        when: ( ContextKeyExpr.and(( CONTEXT_SYNC_STATE.notEqualsTo(SyncStatus.Uninitialized)))),
                    },
                });
            }
            run(accessor) {
                accessor.get(IPreferencesService).openUserSettings({ jsonEditor: false, query: '@tag:sync' });
            }
        }));
    }
    registerHelpAction() {
        const that = this;
        this._register(registerAction2(class HelpAction extends Action2 {
            constructor() {
                super({
                    id: 'workbench.userDataSync.actions.help',
                    title: SYNC_TITLE,
                    category: Categories.Help,
                    menu: [{
                            id: MenuId.CommandPalette,
                            when: ( ContextKeyExpr.and(( CONTEXT_SYNC_STATE.notEqualsTo(SyncStatus.Uninitialized)))),
                        }],
                });
            }
            run() { return that.openerService.open(( URI.parse('https://aka.ms/vscode-settings-sync-help'))); }
        }));
        MenuRegistry.appendMenuItem(MenuId.ViewContainerTitle, {
            command: {
                id: 'workbench.userDataSync.actions.help',
                title: Categories.Help.value
            },
            when: ( ContextKeyExpr.equals('viewContainer', SYNC_VIEW_CONTAINER_ID)),
            group: '1_help',
        });
    }
    registerAcceptMergesAction() {
        const that = this;
        this._register(registerAction2(class AcceptMergesAction extends Action2 {
            constructor() {
                super({
                    id: 'workbench.userDataSync.actions.acceptMerges',
                    title: ( localize(11002, "Complete Merge")),
                    menu: [{
                            id: MenuId.EditorContent,
                            when: ( ContextKeyExpr.and(ctxIsMergeResultEditor, ( ContextKeyExpr.regex(ctxMergeBaseUri.key, ( new RegExp(`^${USER_DATA_SYNC_SCHEME}:`)))))),
                        }],
                });
            }
            async run(accessor, previewResource) {
                const textFileService = accessor.get(ITextFileService);
                await textFileService.save(previewResource);
                const content = await textFileService.read(previewResource);
                await that.userDataSyncService.accept(this.getSyncResource(previewResource), previewResource, content.value, true);
            }
            getSyncResource(previewResource) {
                const conflict = that.userDataSyncService.conflicts.find(({ conflicts }) => ( conflicts.some(conflict => isEqual(conflict.previewResource, previewResource))));
                if (conflict) {
                    return conflict;
                }
                throw ( new Error(`Unknown resource: ${( previewResource.toString())}`));
            }
        }));
    }
    registerDownloadSyncActivityAction() {
        this._register(registerAction2(class DownloadSyncActivityAction extends Action2 {
            constructor() {
                super(DOWNLOAD_ACTIVITY_ACTION_DESCRIPTOR);
            }
            async run(accessor) {
                const userDataSyncWorkbenchService = accessor.get(IUserDataSyncWorkbenchService);
                const notificationService = accessor.get(INotificationService);
                const folder = await userDataSyncWorkbenchService.downloadSyncActivity();
                if (folder) {
                    notificationService.info(( localize(11003, "Successfully downloaded Settings Sync activity.")));
                }
            }
        }));
    }
    registerViews() {
        const container = this.registerViewContainer();
        this.registerDataViews(container);
    }
    registerViewContainer() {
        return ( Registry.as(Extensions.ViewContainersRegistry)).registerViewContainer({
            id: SYNC_VIEW_CONTAINER_ID,
            title: SYNC_TITLE,
            ctorDescriptor: ( new SyncDescriptor(
                ViewPaneContainer,
                [SYNC_VIEW_CONTAINER_ID, { mergeViewWithContainerWhenSingleView: true }]
            )),
            icon: SYNC_VIEW_ICON,
            hideIfEmpty: true,
        }, ViewContainerLocation.Sidebar);
    }
    registerResetSyncDataAction() {
        const that = this;
        this._register(registerAction2(class extends Action2 {
            constructor() {
                super({
                    id: 'workbench.actions.syncData.reset',
                    title: ( localize(11004, "Clear Data in Cloud...")),
                    menu: [{
                            id: MenuId.ViewContainerTitle,
                            when: ( ContextKeyExpr.equals('viewContainer', SYNC_VIEW_CONTAINER_ID)),
                            group: '0_configure',
                        }],
                });
            }
            run() { return that.userDataSyncWorkbenchService.resetSyncedData(); }
        }));
    }
    registerDataViews(container) {
        this._register(this.instantiationService.createInstance(UserDataSyncDataViews, container));
    }
};
UserDataSyncWorkbenchContribution = ( __decorate([
    ( __param(0, IUserDataSyncEnablementService)),
    ( __param(1, IUserDataSyncService)),
    ( __param(2, IUserDataSyncWorkbenchService)),
    ( __param(3, IContextKeyService)),
    ( __param(4, IActivityService)),
    ( __param(5, INotificationService)),
    ( __param(6, IEditorService)),
    ( __param(7, IUserDataProfileService)),
    ( __param(8, IDialogService)),
    ( __param(9, IQuickInputService)),
    ( __param(10, IInstantiationService)),
    ( __param(11, IOutputService)),
    ( __param(12, IUserDataAutoSyncService)),
    ( __param(13, ITextModelService)),
    ( __param(14, IPreferencesService)),
    ( __param(15, ITelemetryService)),
    ( __param(16, IProductService)),
    ( __param(17, IOpenerService)),
    ( __param(18, IAuthenticationService)),
    ( __param(19, IUserDataSyncStoreManagementService)),
    ( __param(20, IHostService)),
    ( __param(21, ICommandService)),
    ( __param(22, IWorkbenchIssueService)),
    ( __param(23, IConfigurationService))
], UserDataSyncWorkbenchContribution));
let UserDataRemoteContentProvider = class UserDataRemoteContentProvider {
    constructor(userDataSyncService, modelService, languageService) {
        this.userDataSyncService = userDataSyncService;
        this.modelService = modelService;
        this.languageService = languageService;
    }
    provideTextContent(uri) {
        if (uri.scheme === USER_DATA_SYNC_SCHEME) {
            return this.userDataSyncService.resolveContent(uri).then(content => this.modelService.createModel(content || '', this.languageService.createById('jsonc'), uri));
        }
        return null;
    }
};
UserDataRemoteContentProvider = ( __decorate([
    ( __param(0, IUserDataSyncService)),
    ( __param(1, IModelService)),
    ( __param(2, ILanguageService))
], UserDataRemoteContentProvider));

export { UserDataSyncWorkbenchContribution };
